class AresponsesAssertionError(AssertionError):
    pass


class NoRouteFoundError(AresponsesAssertionError):
    pass


class UnusedRouteError(AresponsesAssertionError):
    pass


class UnorderedRouteCallError(AresponsesAssertionError):
    pass



import asyncio
import re

import aiohttp
import pytest
import sys
from aiohttp import ServerDisconnectedError

import aresponses as aresponses_mod

# example test in readme.md
from aresponses.errors import (
    NoRouteFoundError,
    UnusedRouteError,
    UnorderedRouteCallError,
)


@pytest.mark.asyncio
async def test_foo(aresponses):
    # text as response (defaults to status 200 response)
    aresponses.add("foo.com", "/", "get", "hi there!!")

    # custom status code response
    aresponses.add("foo.com", "/", "get", aresponses.Response(text="error", status=500))

    # JSON response
    aresponses.add("foo.com", "/", "get", {"status": "ok"})

    # passthrough response (makes an actual network call)
    aresponses.add("httpstat.us", "/200", "get", aresponses.passthrough)

    # custom handler response
    def my_handler(request):
        return aresponses.Response(status=200, text=str(request.url))

    aresponses.add("foo.com", "/", "get", my_handler)

    url = "http://foo.com"
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            text = await response.text()
            assert text == "hi there!!"

        async with session.get(url) as response:
            text = await response.text()
            assert text == "error"
            assert response.status == 500

        async with session.get(url) as response:
            text = await response.text()
            assert text == '{"status": "ok"}'

        async with session.get("https://httpstat.us/200") as response:
            text = await response.text()
        assert text == "200 OK"

        async with session.get(url) as response:
            text = await response.text()
            assert text == "http://foo.com/"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_fixture(aresponses):
    aresponses.add("foo.com", "/", "get", aresponses.Response(text="hi"))

    url = "http://foo.com"
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            text = await response.text()
    assert text == "hi"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_body_match(aresponses):
    aresponses.add(
        "foo.com",
        "/",
        "get",
        aresponses.Response(text="hi"),
        body_pattern=re.compile(r".*?apple.*"),
    )

    url = "http://foo.com"
    async with aiohttp.ClientSession() as session:
        try:
            async with session.get(url, data={"fruit": "pineapple"}) as response:
                text = await response.text()
                assert text == "hi"
        except ServerDisconnectedError:
            pass

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_https(aresponses):
    aresponses.add("foo.com", "/", "get", aresponses.Response(text="hi"))

    url = "https://foo.com"
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            text = await response.text()
    assert text == "hi"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_context_manager():
    loop = asyncio.get_running_loop()
    async with aresponses_mod.ResponsesMockServer(loop=loop) as arsps:
        arsps.add("foo.com", "/", "get", aresponses_mod.Response(text="hi"))

        url = "http://foo.com"
        async with aiohttp.ClientSession() as session:
            async with session.get(url) as response:
                text = await response.text()
        assert text == "hi"

    arsps.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_bad_redirect(aresponses):
    aresponses.add("foo.com", "/", "get", aresponses.Response(text="hi", status=301))
    url = "http://foo.com"
    async with aiohttp.ClientSession() as session:
        response = await session.get(url)
        await response.text()

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_regex(aresponses):
    aresponses.add(
        aresponses.ANY, aresponses.ANY, aresponses.ANY, aresponses.Response(text="hi")
    )
    aresponses.add(
        aresponses.ANY,
        aresponses.ANY,
        aresponses.ANY,
        aresponses.Response(text="there"),
    )

    async with aiohttp.ClientSession() as session:
        async with session.get("http://foo.com") as response:
            text = await response.text()
            assert text == "hi"

        async with session.get("http://bar.com") as response:
            text = await response.text()
            assert text == "there"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_callable(aresponses):
    def handler(request):
        return aresponses.Response(body=request.host)

    aresponses.add(aresponses.ANY, aresponses.ANY, aresponses.ANY, handler)
    aresponses.add(aresponses.ANY, aresponses.ANY, aresponses.ANY, handler)

    async with aiohttp.ClientSession() as session:
        async with session.get("http://foo.com") as response:
            text = await response.text()
            assert text == "foo.com"

        async with session.get("http://bar.com") as response:
            text = await response.text()
            assert text == "bar.com"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_raw_response(aresponses):
    raw_response = b"""HTTP/1.1 200 OK\r
Date: Tue, 26 Dec 2017 05:47:50 GMT\r
\r
<html><body><h1>It works!</h1></body></html>
"""
    aresponses.add(
        aresponses.ANY,
        aresponses.ANY,
        aresponses.ANY,
        aresponses.RawResponse(raw_response),
    )

    async with aiohttp.ClientSession() as session:
        async with session.get("http://foo.com") as response:
            text = await response.text()
            assert "It works!" in text

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_querystring(aresponses):
    aresponses.add("foo.com", "/path", "get", aresponses.Response(text="hi"))

    url = "http://foo.com/path?reply=42"
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            text = await response.text()
    assert text == "hi"

    aresponses.add(
        "foo.com",
        "/path2?reply=42",
        "get",
        aresponses.Response(text="hi"),
        match_querystring=True,
    )

    url = "http://foo.com/path2?reply=42"
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            text = await response.text()
    assert text == "hi"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_querystring_not_match(aresponses):
    aresponses.add(
        "foo.com",
        "/path",
        "get",
        aresponses.Response(text="hi"),
        match_querystring=True,
    )
    aresponses.add(
        "foo.com",
        aresponses.ANY,
        "get",
        aresponses.Response(text="miss"),
        match_querystring=True,
    )
    aresponses.add(
        "foo.com",
        aresponses.ANY,
        "get",
        aresponses.Response(text="miss"),
        match_querystring=True,
    )

    url = "http://foo.com/path"
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            text = await response.text()
    assert text == "hi"

    url = "http://foo.com/path?reply=42"
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            text = await response.text()
    assert text == "miss"

    url = "http://foo.com/path?reply=43"
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            text = await response.text()
    assert text == "miss"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_passthrough(aresponses):
    aresponses.add("httpstat.us", "/200", "get", aresponses.passthrough)

    url = "https://httpstat.us/200"
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            text = await response.text()
    assert text == "200 OK"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_failure_not_called(aresponses):
    aresponses.add("foo.com", "/", "get", aresponses.Response(text="hi"))
    with pytest.raises(UnusedRouteError):
        aresponses.assert_no_unused_routes()

    with pytest.raises(UnusedRouteError):
        aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_failure_no_match(aresponses):
    async with aiohttp.ClientSession() as session:
        try:
            async with session.get("http://foo.com") as response:
                await response.text()
        except ServerDisconnectedError:
            pass
    with pytest.raises(NoRouteFoundError):
        aresponses.assert_all_requests_matched()

    with pytest.raises(NoRouteFoundError):
        aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_failure_bad_ordering(aresponses):
    aresponses.add("foo.com", "/a", "get", aresponses.Response(text="hi"))
    aresponses.add("foo.com", "/b", "get", aresponses.Response(text="hi"))

    async with aiohttp.ClientSession() as session:
        async with session.get("http://foo.com/b") as response:
            await response.text()
        async with session.get("http://foo.com/a") as response:
            await response.text()

    aresponses.assert_all_requests_matched()
    aresponses.assert_no_unused_routes()
    with pytest.raises(UnorderedRouteCallError):
        aresponses.assert_called_in_order()

    with pytest.raises(UnorderedRouteCallError):
        aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_failure_not_called_enough_times(aresponses):
    aresponses.add("foo.com", "/", "get", aresponses.Response(text="hi"), repeat=2)

    async with aiohttp.ClientSession() as session:
        async with session.get("http://foo.com/") as response:
            await response.text()

    with pytest.raises(UnusedRouteError):
        aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_history(aresponses):
    aresponses.add(response=aresponses.Response(text="hi"), repeat=2)

    async with aiohttp.ClientSession() as session:
        async with session.get("http://foo.com/b") as response:
            await response.text()
        async with session.get("http://bar.com/a") as response:
            await response.text()

    assert len(aresponses.history) == 2
    assert aresponses.history[0].request.host == "foo.com"
    assert aresponses.history[1].request.host == "bar.com"
    assert "Route(" in repr(aresponses.history[0].route)
    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_history_post(aresponses):
    """Ensure the request contents exist in the history"""
    aresponses.add(method_pattern="POST", response={"some": "response"})

    async with aiohttp.ClientSession() as session:
        async with session.post(
            "http://bar.com/zzz", json={"greeting": "hello"}
        ) as response:
            response_data = await response.json()
            assert response_data == {"some": "response"}

    assert len(aresponses.history) == 1
    assert aresponses.history[0].request.host == "bar.com"
    request_data = await aresponses.history[0].request.json()
    assert request_data == {"greeting": "hello"}
    assert "Route(" in repr(aresponses.history[0].route)
    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_history_post_binary(aresponses):
    """
    Ensure the request contents exist in the history

    ...and that it can handle binary requests
    """
    binary_not_utf = b"o\xad<|6\xd2a\x116\x17\xdb\x98-60:"
    aresponses.add(method_pattern="POST", response={"some": "response"})

    async with aiohttp.ClientSession() as session:
        async with session.post("http://bar.com/zzz", data=binary_not_utf) as response:
            response_data = await response.json()
            assert response_data == {"some": "response"}

    assert len(aresponses.history) == 1
    assert aresponses.history[0].request.host == "bar.com"
    request_data = await aresponses.history[0].request.read()
    assert request_data == binary_not_utf


@pytest.fixture()
def _short_recursion_limit():
    # The default is 1000, but it takes time (seconds); 100 is much faster.
    old_limit = sys.getrecursionlimit()
    sys.setrecursionlimit(100)
    yield
    sys.setrecursionlimit(old_limit)


@pytest.mark.asyncio
@pytest.mark.usefixtures("_short_recursion_limit")
async def test_not_exceeding_recursion_limit():
    loop = asyncio.get_running_loop()
    for _ in range(sys.getrecursionlimit()):
        async with aresponses_mod.ResponsesMockServer(loop=loop) as arsps:
            arsps.add("fake-host", "/", "get", "hello")
            async with aiohttp.ClientSession() as session:
                async with session.get("http://fake-host"):
                    pass



import re

ANY = re.compile(".*")


def _text_matches_pattern(pattern, text):
    # This is needed for compatibility with old Python versions
    try:
        pattern_class = re.Pattern
    except AttributeError:
        pattern_class = re._pattern_type
    if isinstance(pattern, str):
        if pattern == text:
            return True
    elif isinstance(pattern, pattern_class):
        if pattern.search(text):
            return True
    return False



import math
import re

import aiohttp
import pytest


@pytest.mark.asyncio
async def test_simple(aresponses):
    aresponses.add("google.com", "/api/v1/", "GET", response="OK")
    aresponses.add("foo.com", "/", "get", aresponses.Response(text="error", status=500))

    async with aiohttp.ClientSession() as session:
        async with session.get("http://google.com/api/v1/") as response:
            text = await response.text()
            assert text == "OK"

        async with session.get("https://foo.com") as response:
            text = await response.text()
            assert text == "error"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_regex_repetition(aresponses):
    aresponses.add(re.compile(r".*\.?google\.com"), response="OK", repeat=2)

    async with aiohttp.ClientSession() as session:
        async with session.get("http://google.com") as response:
            text = await response.text()
            assert text == "OK"

        async with session.get("http://api.google.com") as response:
            text = await response.text()
            assert text == "OK"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_json(aresponses):
    aresponses.add("google.com", "/api/v1/", "GET", response={"status": "OK"})

    async with aiohttp.ClientSession() as session:
        async with session.get("http://google.com/api/v1/") as response:
            assert {"status": "OK"} == await response.json()

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_handler(aresponses):
    def break_everything(request):
        return aresponses.Response(status=500, text=str(request.url))

    aresponses.add(response=break_everything, repeat=math.inf)

    async with aiohttp.ClientSession() as session:
        async with session.get("http://google.com/api/v1/") as response:
            assert response.status == 500



import asyncio
import logging
import math
import re
from copy import copy
from typing import List, NamedTuple

try:
    from pytest_asyncio import fixture as asyncio_fixture
except ImportError:
    # Backward compatability for pytest-asyncio<0.17
    import pytest

    asyncio_fixture = pytest.fixture

from aiohttp import web, ClientSession
from aiohttp.client_reqrep import ClientRequest
from aiohttp.connector import TCPConnector
from aiohttp.helpers import sentinel
from aiohttp.test_utils import BaseTestServer
from aiohttp.web_request import BaseRequest
from aiohttp.web_response import StreamResponse, json_response
from aiohttp.web_runner import ServerRunner
from aiohttp.web_server import Server

from aresponses.errors import (
    NoRouteFoundError,
    UnusedRouteError,
    UnorderedRouteCallError,
)
from aresponses.utils import _text_matches_pattern, ANY

logger = logging.getLogger(__name__)


class RawResponse(StreamResponse):
    """
    Allow complete control over the response

    Useful for mocking invalid responses
    """

    def __init__(self, body):
        super().__init__()
        self._body = body

    async def _start(self, request, *_, **__):
        self._req = request
        self._keep_alive = False
        writer = self._payload_writer = request._payload_writer
        return writer

    async def write_eof(self, *_, **__):  # noqa
        await super().write_eof(self._body)


class Route:
    def __init__(
        self,
        method_pattern=ANY,
        host_pattern=ANY,
        path_pattern=ANY,
        body_pattern=ANY,
        match_querystring=False,
        repeat=1,
    ):
        self.method_pattern = method_pattern
        self.host_pattern = host_pattern
        self.path_pattern = path_pattern
        self.body_pattern = body_pattern
        self.match_querystring = match_querystring
        self.repeat = repeat

    async def matches(self, request):
        path_to_match = request.path_qs if self.match_querystring else request.path

        if not _text_matches_pattern(self.host_pattern, request.host):
            return False

        if not _text_matches_pattern(self.path_pattern, path_to_match):
            return False

        if not _text_matches_pattern(self.method_pattern, request.method.lower()):
            return False

        if self.body_pattern != ANY:
            if not _text_matches_pattern(self.body_pattern, await request.text()):
                return False

        return True

    def __str__(self):
        return (
            f"method={self.method_pattern} host_pattern={self.host_pattern} "
            f"path={self.path_pattern} body={self.body_pattern} "
            f"match_querystring={self.match_querystring}"
        )

    def __repr__(self):
        return (
            f"Route(method={repr(self.method_pattern)}, "
            f"host_pattern={repr(self.host_pattern)}, "
            f"path={repr(self.path_pattern)}, "
            f"body={repr(self.body_pattern)}, "
            f"match_querystring={repr(self.match_querystring)})"
        )


class RoutingLog(NamedTuple):
    request: BaseRequest
    route: Route
    response: StreamResponse


class ResponsesMockServer(BaseTestServer):
    ANY = ANY
    Response = web.Response
    RawResponse = RawResponse
    INFINITY = math.inf
    LOCALHOST = re.compile(r"127\.0\.0\.1:?\d{0,5}")

    def __init__(self, *, scheme=sentinel, host="127.0.0.1", **kwargs):
        self._responses = []
        self._exception = None
        self._unmatched_requests = []
        self._first_unordered_route = None
        self._request_count = 0
        self._history = []
        super().__init__(scheme=scheme, host=host, **kwargs)

    async def _make_runner(self, debug=True, **kwargs):
        srv = Server(self._handler, loop=self._loop, debug=True, **kwargs)
        return ServerRunner(srv, debug=debug, **kwargs)

    async def _handler(self, request):
        self._request_count += 1
        route, response = await self._find_response(request)
        # ensures the request content is loaded even if the handler didn't
        # need it. This makes it available in`aresponses.history`
        await request.read()
        self._history.append(RoutingLog(request, route, response))
        return response

    def add(
        self,
        host_pattern=ANY,
        path_pattern=ANY,
        method_pattern=ANY,
        response="",
        *,
        route=None,
        body_pattern=ANY,
        match_querystring=False,
        repeat=1,
    ):
        """
        Adds a route and response to the mock server.

        When the route is hit `repeat` times it will be removed from the routing table.Z

        :param host_pattern:
        :param path_pattern:
        :param method_pattern:
        :param response:
        :param route: A Route object.  Overrides all args except for `response`.
                        Useful for custom matching.
        :param body_pattern:
        :param match_querystring:
        :param repeat:
        :return:
        """
        if isinstance(host_pattern, str):
            host_pattern = host_pattern.lower()

        if isinstance(method_pattern, str):
            method_pattern = method_pattern.lower()

        if route is None:
            route = Route(
                method_pattern=method_pattern,
                host_pattern=host_pattern,
                path_pattern=path_pattern,
                body_pattern=body_pattern,
                match_querystring=match_querystring,
                repeat=repeat,
            )

        self._responses.append((route, response))

    def add_local_passthrough(self, repeat=INFINITY):
        self.add(host_pattern=self.LOCALHOST, repeat=repeat, response=self.passthrough)

    async def _find_response(self, request):
        for i, (route, response) in enumerate(self._responses):
            if not await route.matches(request):
                continue

            route.repeat -= 1

            if route.repeat <= 0:
                del self._responses[i]
            else:
                self._responses[i] = (route, copy(response))

            if callable(response):
                if asyncio.iscoroutinefunction(response):
                    response = await response(request)
                else:
                    response = response(request)

            elif isinstance(response, str):
                response = self.Response(body=response)
            elif isinstance(response, (dict, list)):
                response = json_response(data=response)

            if i > 0 and self._first_unordered_route is None:
                self._first_unordered_route = route

            return route, response

        self._unmatched_requests.append(request)
        return None, None

    async def passthrough(self, request):
        """Make non-mocked network request"""

        class DirectTcpConnector(TCPConnector):
            def _resolve_host(slf, *args, **kwargs):  # noqa
                return self._old_resolver_mock(slf, *args, **kwargs)

        class DirectClientRequest(ClientRequest):
            def is_ssl(slf) -> bool:  # noqa
                return slf._aresponses_direct_is_ssl()

        connector = DirectTcpConnector()

        original_request = request.clone(
            scheme="https" if request.headers["AResponsesIsSSL"] else "http"
        )

        headers = {k: v for k, v in request.headers.items() if k != "AResponsesIsSSL"}

        async with ClientSession(
            connector=connector, request_class=DirectClientRequest
        ) as session:
            request_method = getattr(session, request.method.lower())
            async with request_method(
                original_request.url, headers=headers, data=(await request.read())
            ) as r:
                headers = {
                    k: v for k, v in r.headers.items() if k.lower() == "content-type"
                }
                data = await r.read()
                response = self.Response(body=data, status=r.status, headers=headers)
                return response

    async def __aenter__(self) -> "ResponsesMockServer":
        await self.start_server(loop=self._loop)

        self._old_resolver_mock = TCPConnector._resolve_host

        async def _resolver_mock(_self, host, port, traces=None):
            return [
                {
                    "hostname": host,
                    "host": "127.0.0.1",
                    "port": self.port,
                    "family": _self._family,
                    "proto": 0,
                    "flags": 0,
                }
            ]

        TCPConnector._resolve_host = _resolver_mock

        self._old_is_ssl = ClientRequest.is_ssl
        ClientRequest._aresponses_direct_is_ssl = ClientRequest.is_ssl

        def new_is_ssl(_self):
            return False

        ClientRequest.is_ssl = new_is_ssl

        # store whether a request was an SSL request in the `AResponsesIsSSL` header
        self._old_init = ClientRequest.__init__

        def new_init(_self, *largs, **kwargs):
            self._old_init(_self, *largs, **kwargs)

            is_ssl = "1" if self._old_is_ssl(_self) else ""
            _self.update_headers({**_self.headers, "AResponsesIsSSL": is_ssl})

        ClientRequest.__init__ = new_init

        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        TCPConnector._resolve_host = self._old_resolver_mock
        ClientRequest.is_ssl = self._old_is_ssl
        ClientRequest.__init__ = self._old_init

        await self.close()

    def assert_no_unused_routes(self, ignore_infinite_repeats=False):
        for route, _ in self._responses:
            if not ignore_infinite_repeats or route.repeat != self.INFINITY:
                raise UnusedRouteError(f"Unused Route: {route}")

    def assert_called_in_order(self):
        if self._first_unordered_route is not None:
            raise UnorderedRouteCallError(
                f"Route used out of order: {self._first_unordered_route}"
            )

    def assert_all_requests_matched(self):
        if self._unmatched_requests:
            request = self._unmatched_requests[0]
            raise NoRouteFoundError(
                f"No match found for request: "
                f"{request.method} {request.host} {request.path}"
            )

    def assert_plan_strictly_followed(self):
        self.assert_no_unused_routes()
        self.assert_called_in_order()
        self.assert_all_requests_matched()

    @property
    def history(self) -> List[RoutingLog]:
        return self._history


@asyncio_fixture()
async def aresponses() -> ResponsesMockServer:
    loop = asyncio.get_running_loop()
    async with ResponsesMockServer(loop=loop) as server:
        yield server



import aiohttp
import pytest
from aiohttp import web


@pytest.fixture()
def loop(event_loop):
    """replace aiohttp loop fixture with pytest-asyncio fixture"""
    return event_loop


def make_app():
    app = web.Application()

    async def constant_handler(request):
        return web.Response(text="42")

    async def ip_handler(request):
        protocol = request.query["protocol"]
        ip = await get_ip_address(protocol=protocol)
        return web.Response(text=f"ip is {ip}")

    app.add_routes([web.get("/constant", constant_handler)])
    app.add_routes([web.get("/ip", ip_handler)])
    return app


async def get_ip_address(protocol):
    async with aiohttp.ClientSession() as s:
        async with s.get(f"{protocol}://httpbin.org/ip") as resp:
            ip = (await resp.json())["origin"]
            return ip


@pytest.mark.asyncio
async def test_app_simple_endpoint(aiohttp_client):
    client = await aiohttp_client(make_app())
    r = await client.get("/constant")
    assert (await r.text()) == "42"


@pytest.mark.asyncio
async def test_app_simple_endpoint_with_aresponses(aiohttp_client, aresponses):
    """
    when testing your own aiohttp server you must setup passthrough to it

    Ideally this wouldn't be necessary but haven't figured that out yet.
    Perhaps all local calls should be passthrough.
    """
    aresponses.add("127.0.0.1:4241", response=aresponses.passthrough)

    client = await aiohttp_client(make_app(), server_kwargs={"port": 4241})
    r = await client.get("/constant")
    assert (await r.text()) == "42"


@pytest.mark.asyncio
@pytest.mark.parametrize("protocol", ["http", "https"])
async def test_app_with_subrequest_using_aresponses(
    aiohttp_client, aresponses, protocol
):
    """
    but passthrough doesn't work if the handler itself makes an aiohttp https request
    """
    aresponses.add_local_passthrough(repeat=1)
    aresponses.add("httpbin.org", response={"origin": "1.2.3.4"})

    client = await aiohttp_client(make_app())
    r = await client.get(f"/ip?protocol={protocol}")
    body = await r.text()
    assert r.status == 200, body
    assert "ip is" in (await r.text())
    aresponses.assert_plan_strictly_followed()



from os import path

from setuptools import setup

__version__ = "3.0.0"

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="aresponses",
    packages=["aresponses"],
    version=__version__,
    description="""
    Asyncio response mocking. Similar to the responses library used for 'requests'
    """.strip(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Bryce Drennan",
    author_email="aresponses@brycedrennan.com   ",
    url="https://github.com/aresponses/aresponses",
    download_url="https://github.com/aresponses/aresponses/tarball/" + __version__,
    keywords=["asyncio", "testing", "responses"],
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires=">=3.7",
    install_requires=[
        'aiohttp<3.9.0,>=3.1.0; python_version>="3.7" and python_version<"3.8"',
        'aiohttp>=3.6.0; python_version>="3.8" and python_version<"3.10"',
        'aiohttp>=3.7.0; python_version>="3.10" and python_version<"3.12"',
        'aiohttp>=3.7.0,!=3.8.*; python_version>="3.12"',
        'pytest-asyncio==0.16.0; python_version<"3.7"',
        'pytest-asyncio>=0.17.0; python_version>="3.7"',
    ],
    # the following makes a plugin available to pytest
    entry_points={"pytest11": ["aresponses = aresponses.main"]},
)
